# Basic correct solution to Part 3
# No extra credit.

import errno
import web
import cgi
import web.form as f
import cPickle
from datetime import datetime

MESSAGE_DB = 'msg.db'

urls = (
    '/',    'Timeline',
    '/new', 'NewPost',
    )

class Message(object):
    def __init__(self, text, user="Anonymous Coward"):
        self.text = text
        self.user = user
        self.when = datetime.now()

    def __str__(self):
        return u"%s, %s, %s" % (self.text, self.user, self.when)

    def ago(self):
        delta = datetime.now() - self.when
        if delta.days > 1:
            return "%d days ago" % delta.days
        if delta.days == 1:
            return "yesterday"
        elif 7200 > delta.seconds >= 3600:
            return "last hour"
        elif delta.seconds >= 3600:
            return "%d hours ago" % (delta.seconds / 3600)
        elif 120 > delta.seconds >= 60:
            return "last minute"
        elif delta.seconds > 60:
            return "%d minutes ago" % (delta.seconds / 60)
        else:
            return "just about now"

    def to_html(self):
        return "<p><b>%s</b>: %s <i>&mdash; %s</i> </p>" % \
            (cgi.escape(self.user),
             cgi.escape(self.text),
             cgi.escape(self.ago()))


class MyPage(object):
    html = """
<html>
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <title>Insipid Insights</title>
  </head>
  <body>
    <h1>Insipid Insights</h1>
    <blockquote> <i> &#8220;I was never more annoyed! The insipidity,
          and yet the noise&mdash;the nothingness,
          and yet the self-importance of all those people!&#8221; </i>
      <br>&mdash; Jane Austin, <a href="http://www.gutenberg.org/etext/1342">Pride and Prejudice</a></blockquote>
    <hr>
    %s <!-- content -->
    <hr>
    <p>%s | This is a demo for <a href="http://www.cs.unc.edu/Courses/comp524-s10/">COMP 524</a>. Be nice.</p>
  </body>
</html>
"""

class Timeline(MyPage):
    def GET(self):
        return self.html % ("\n".join([msg.to_html() for msg in web.ctx.messages]),
                       '<b>Pondering Timeline</b> | <a href="new">Think Hard</a>')

class NewPost(MyPage):
    form = f.Form(
        f.Textbox('user', description="My name is ", size=25),
        f.Textarea('message', description="I realized ", cols=50, rows=3),
        f.Button('Add Insight', post=' Limit: 140 characters, for no good reason.'),
        )

    def GET(self):
        content = '<form name="main" method="post" accept-charset="utf-8">' \
            '%s</form>'
        return self.html % (content % self.form().render(),
                            '<a  href="/">Ponder Timeline</a> | <b>Thinking Hard</b>')

    def POST(self):
        data = self.form()
        try:
            data.fill()
            msg = data.d.message
            usr = data.d.user
        except UnicodeDecodeError:
            # bad input, ignore
            msg = None
            usr = None
        if msg:
            msg.strip()
            # truncate if too long
            if len(msg) > 140:
                words = msg.split()
                sep   =  u" [...] "
                msg   = u""
                while words \
                        and len(msg) + len(words[0]) <= 140 - len(words[-1]):
                    msg += u" " + words.pop(0)
                if words:
                    msg += sep
                    msg += words[-1]
            if usr:
                usr = data.d.user[:25]
            else:
                usr = str(web.ctx.ip)
            posted = Message(msg, usr)
            web.ctx.messages.append(posted)
            print "[%s] " % web.ctx.ip,
            print posted.user.encode('utf8'),
            print ">>",
            print posted.text.encode('utf8'),
            print "at",
            print posted.when
            store_messages(web.ctx.messages)
        return web.seeother('/')

def load_messages():
    try:
        f = open(MESSAGE_DB, 'r')
        msgs =  cPickle.load(f)
        f.close()
        return msgs
    except IOError, ex:
        if ex.errno != errno.ENOENT:
            print "Message DB error:", ex
        else:
            # ok if the file is missing
            return [Message("Welcome to Insipid Insights---enjoy!", "bbb")]

def store_messages(msgs):
    f = open(MESSAGE_DB, 'w')
    cPickle.dump(msgs, f)
    f.close()

if __name__ == '__main__':
    all_messages = load_messages()
    for x in all_messages:
        print "Loaded %s: %s (%s)" % (x.user, x.text, x.ago())
    app = web.application(urls, globals())
    app.internalerror = web.debugerror
    def proc(handle):
        web.ctx.messages = all_messages
        return handle()
    app.add_processor(proc)
    app.run()
    store_messages(all_messages)

