from optparse import OptionParser, make_option as o
from webbrowser import open_new_tab
import feedparser
import sys
import re

options = [
    o("-s", "--no-summary", action="store_true", default=False,
      dest="no_summary", help="do not show summary"),
    o("-n", "--max-items", action="store", type="int",
      dest="max_items", help="limit output to MAX_ITEMS per feed"),
    o("-o", "--open-browser", action="store_true", default=False,
      dest="open_browser", help="open link in browser"),
]

defaults = {
    "no_summary"   : False,
    "max_items"    : None,
    "open_browser" : False,
}

def convert_select_html(str):
    return re.sub("<[bB][rR] */?>", "\n", str)

def print_indented(str, indent=3, columns=80, out=sys.stdout):
    lines = str.split('\n')
    cur_pos = indent
    istr    = " " * indent
    out.write(istr)
    for line in lines:
        for word in line.split():
            lw = len(word)
            if lw + cur_pos > columns:
                out.write("\n%s" % istr)
                cur_pos = indent
            out.write(word)
            out.write(" ")
            cur_pos += lw + 1

def show_entry(opts, entry):
    "Display one entry. Checks for missing fields."
    title  = entry.title if entry.has_key('title') else "<untitled>"
    author = entry.author if entry.has_key('author') else "<anonymous>"
    date   = " on %s" % entry.date if entry.has_key('date') else ""
    link   = "\n   URL: %s" % entry.link if entry.has_key('link') else ""
    print "-- %s\n   by %s%s%s" % (title, author, date, link)
    if not opts.no_summary and entry.has_key('description'):
        print
        print_indented(convert_select_html(entry.description))
    print "\n"

    if entry.has_key('link') and opts.open_browser:
        open_new_tab(entry.link)

def fetch_and_show(opts, url):
    d = feedparser.parse(url)
    entries = d.entries
    if opts.max_items and opts.max_items > 0:
        entries = entries[0:opts.max_items]
    title = "Results for %s" % d.feed.title if d.feed.has_key('title') else url
    print title
    print "-" * len(title)
    if entries:
        for entry in entries:
            show_entry(opts, entry)
    elif not d.bozo:
        print "<no results>"
    else:
        print "Sorry, the document could not be retrieved and parsed."
    print "\n"

def rss_main(args=sys.argv):
    oparse = OptionParser(option_list=options)
    (opts, args) = oparse.parse_args(args=args)
    for url in args[1:]:
        fetch_and_show(opts, url)


if __name__ == "__main__":
    rss_main()

